@@ -17,7 +17,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
-
 import os
 import time
 import logging
@@ -29,7 +28,7 @@ from pyanaconda.flags import flags
 from pyanaconda.packaging import ImagePayload, PayloadInstallError
 from pyanaconda.threads import threadMgr, AnacondaThread
 from pyanaconda.i18n import _
-from pyanaconda.constants import ROOT_PATH, INSTALL_TREE, THREAD_LIVE_PROGRESS
+from pyanaconda.constants import INSTALL_TREE, THREAD_LIVE_PROGRESS
 from pyanaconda.progress import progressQ
 
 from blivet.size import Size
@@ -45,7 +44,7 @@ class LiveCDCopyBackend(ImagePayload):
     def __init__(self, *args, **kwargs):
         super(LiveCDCopyBackend, self).__init__(*args, **kwargs)
 
-        # Used to adjust size of ROOT_PATH when files are already present
+        # Used to adjust size of iutil.getSysroot() when files are already present
         self._adj_size = 0
         self.pct = 0
         self.pct_lock = None
@@ -65,18 +64,23 @@ class LiveCDCopyBackend(ImagePayload):
 
         return self._entropy_prop
 
+    @property
+    def needsNetwork(self):
+        return True
+
     @property
     def kernelVersionList(self):
         vers = []
-        for name in os.listdir("/boot"):
+        boot_dir = os.path.join(iutil.getSysroot(), "boot")
+        for name in os.listdir(boot_dir):
             if name.startswith("kernel-genkernel-"):
                 vers.append(name[len("kernel-genkernel-"):])
         return vers
 
     @property
     def spaceRequired(self):
-        return Size(bytes=iutil.getDirSize(
-                os.path.realpath(INSTALL_TREE)) * 1024)
+        return Size(iutil.getDirSize(
+            os.path.realpath(INSTALL_TREE)) * 1024)
 
     def recreateInitrds(self, force=False):
         log.info("calling recreateInitrds()")
@@ -100,8 +104,8 @@ class LiveCDCopyBackend(ImagePayload):
     def repos(self):
         return self.entropy.repositories()
 
-    def setup(self, storage):
-        super(LiveCDCopyBackend, self).setup(storage)
+    def setup(self, storage, instClass):
+        super(LiveCDCopyBackend, self).setup(storage, instClass)
 
     def progress(self):
         """Monitor the amount of disk space used on the target and source and
@@ -112,7 +116,7 @@ class LiveCDCopyBackend(ImagePayload):
         while self.pct < 100:
             dest_size = 0
             for mnt in mountpoints:
-                mnt_stat = os.statvfs(ROOT_PATH+mnt)
+                mnt_stat = os.statvfs(iutil.getSysroot()+mnt)
                 dest_size += mnt_stat.f_frsize * (mnt_stat.f_blocks - mnt_stat.f_bfree)
             if dest_size >= self._adj_size:
                 dest_size -= self._adj_size
@@ -153,7 +157,7 @@ class LiveCDCopyBackend(ImagePayload):
         # file system boundaries
         args = ["-pogAXtlHrDx", "--exclude", "/dev/", "--exclude", "/proc/",
                 "--exclude", "/sys/", "--exclude", "/run/",
-                "--exclude", "/etc/machine-id", INSTALL_TREE+"/", ROOT_PATH]
+                "--exclude", "/etc/machine-id", INSTALL_TREE+"/", iutil.getSysroot()]
         try:
             rc = iutil.execWithRedirect(cmd, args)
         except (OSError, RuntimeError) as e:
@@ -177,42 +181,39 @@ class LiveCDCopyBackend(ImagePayload):
 
     def _setDefaultBootTarget(self):
         """ Set the default systemd target for the system. """
-        # If X was already requested we don't have to continue
-        if self.data.xconfig.startX:
+        default_target = INSTALL_TREE + '/etc/systemd/system/default.target'
+        # If we're using VNC, we probably don't want X.
+        if flags.usevnc:
+            self.data.xconfig.startX = False
             return
 
-        if not flags.usevnc:
-            # We only manipulate the ksdata.  The symlink is made later
-            # during the config write out.
-            self.data.xconfig.startX = True
+        # Otherwise, let's assume that we want the same setup as
+        # the live system.
+        if os.path.isfile(default_target):
+            try:
+                target = os.readlink(default_target)
+            except (OSError, IOError) as err:
+                log.error('Unable to read %s: %s', default_target, err)
+            else:
+                target_name = os.path.basename(target)
+                start_x = target_name == 'graphical.target'
+                self.data.xconfig.startX = start_x
+
+                log.info('Current boot target name is %s, starting x: %s',
+                         target_name, start_x)
+        else:
+            log.error('%s does not exist, unable to set boot mode.',
+                      default_target)
+
+    @property
+    def bootArgsList(self):
+        """Get boot arguments."""
+        return self._sabayon_install.get_boot_args()
 
     def postInstall(self):
-        super(LiveCDCopyBackend, self).postInstall()
-
         log.info("Preparing to configure Sabayon (backend postInstall)")
-
-        self._sabayon_install.spawn_chroot(
-            ["/usr/bin/systemd-machine-id-setup"]
-            )
-        self._sabayon_install.setup_secureboot()
-        self._sabayon_install.setup_sudo()
-        self._sabayon_install.remove_proprietary_drivers()
-        self._sabayon_install.setup_nvidia_legacy()
-        self._sabayon_install.configure_skel()
-        self._sabayon_install.configure_services()
-        self._sabayon_install.spawn_chroot(["env-update"])
-        self._sabayon_install.spawn_chroot(["ldconfig"])
-
-        if self._packages:
-            log.info("Preparing to install these packages: %s" % (
-                    self._packages,))
-            self._sabayon_install.setup_entropy_mirrors()
-            self._sabayon_install.maybe_install_packages(self._packages)
-
-        self._sabayon_install.configure_boot_args()
-
+        self._setDefaultBootTarget()
         self._sabayon_install.emit_install_done()
-
         progressQ.send_message(_("Sabayon configuration complete"))
 
     def configure(self):
@@ -230,12 +231,23 @@ class LiveCDCopyBackend(ImagePayload):
                     err,))
             username = "root"  # if no admin user was created
         self._sabayon_install.configure_steambox(username)
+        self._sabayon_install.configure_admin_user_groups(username)
 
-        # also remove hw.hash
-        hwhash_file = os.path.join(ROOT_PATH, "etc/entropy/.hw.hash")
-        try:
-            os.remove(hwhash_file)
-        except (OSError, IOError):
-            pass
-
+        self._sabayon_install.remove_hwhash()
+        self._sabayon_install.setup_machine_id()
         self._sabayon_install.cleanup_packages()
+        self._sabayon_install.setup_boot()
+        self._sabayon_install.setup_secureboot()
+        self._sabayon_install.setup_sudo()
+        self._sabayon_install.remove_proprietary_drivers()
+        self._sabayon_install.setup_nvidia_legacy()
+        self._sabayon_install.configure_skel()
+        self._sabayon_install.configure_services()
+        self._sabayon_install.spawn_chroot(["env-update"])
+        self._sabayon_install.spawn_chroot(["ldconfig"])
+
+        if self._packages:
+            log.info("Preparing to install these packages: %s" % (
+                    self._packages,))
+            self._sabayon_install.setup_entropy_mirrors()
+            self._sabayon_install.maybe_install_packages(self._packages)
